// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Agent.java
// The main characters in the game. This class will eventuallyl be inherited 
// by the human or computer agent classes.
// ------------------------------------------------------------------------------

class Agent extends PhysicsModel
{
  float accelK;
  float turnK;
  float brakeK;
  float dampK;
  float smallestModelRadius;

  /**
   * Constructor
   */
  Agent(String N, Polygon P, Material M, Point C, float R, Settings S, String I)
  {
    super(N, "Agent", P, M, C, R, S, I);
  }
  
  /**
   * Moves the agent according to the input variables
   */
  void drive(boolean accelerate, boolean brake, boolean turnLeft, boolean turnRight, String type)
  {
    Point c = new Point(0,0);                                // Center position
    Point b = new Point(0,8).rotate(poly.rotation);          // Rear exhaust position
    Point f = new Point(0,-8).rotate(poly.rotation);         // Front tire position
    Point r = new Point(-5,8).rotate(poly.rotation);         // Right wing tip
    Point l = new Point(+5,8).rotate(poly.rotation);         // Left wing tip
    
    Force cfNormal = new Force(c, new Vector(new Point(0,-1).rotate(poly.rotation)));       // Acceleration force with length 1
    Force ffNormal = new Force(f, new Vector(new Point(-1,0).rotate(poly.rotation)));       // Front steering left force with length 1
    
    if (accelerate)
    {
      forces.add(cfNormal.multiply(accelK));
      emitParticles("Accel" + type, 0, new Vector(center).add(b), cfNormal.vector.multiply(-2), 4);
    }
    if (brake)
    {
      forces.add(cfNormal.multiply(-brakeK));
    }
    if (turnLeft)
    {
      forces.add(ffNormal.multiply(turnK));
    }
    if (turnRight)
    {
      forces.add(ffNormal.multiply(-turnK));
    }
  }
  
  /**
   * Parse the input string
   */
  void parseInput(String input)
  {     
    String[] args = inputToArray(input);
     
    accelK = Str2Int(args[0]);
    turnK = Str2Flt(args[1]);
    brakeK = Str2Flt(args[2]);
    dampK = Str2Flt(args[3]);
  }
}
