// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// DataLoader.java
// This class will make loading data from the disk easy.
// ------------------------------------------------------------------------------

class DataLoader
{
  /**
   * Load an nxn matrix from disk
   */
  String[][] loadData(String location, String separator)
  {
    String[] data = loadStrings(location);
    
    if (data.length != 0)
    {
      String[] tokens = data[0].split(separator);
    
      // Assumes that each line has the same number of tokens
      String[][] R = new String[data.length][tokens.length];
    
      // Fill up R.
      for (int i=0; i<data.length; i++)
      {
        for (int j=0; j<tokens.length; j++)
        {
          R[i][j] = data[i].split(separator)[j];
        }
      }
      
      return R;
    }
    else
    {
      return new String[0][0];
    }
  }
  
  /**
   * Load an nx1 matrix from disk
   */
  String[] loadData(String location)
  {
    String[] data = loadStrings(location);
    
    // Assumes that each line has the same number of tokens
    String[] R = new String[data.length];
    
    // Fill up R.
    for (int i=0; i<data.length; i++)
    {
        R[i] = data[i];
    }
    
    return R;
  }
  
  /**
   * Preset function that loads polygon information
   * from disk
   */
  Polygon loadPolygon(String P)
  {
    // The polygon that gets returned
    Polygon R = new Polygon();
    
    // Each line represents one point in the polygon
    String[][] polyData = loadData("data/polygons/" + P + ".aip", ",");
    for (int j=0; j<polyData.length; j++)
    {
      float x = Str2Flt(polyData[j][0]);
      float y = Str2Flt(polyData[j][1]);
      R.insertVertex(new Point(x,y));
    }
    
    return R;
  }
  
  /**
   * Preset function that loads material information
   * from disk
   */
  Material loadMaterial(String M)
  {
    // The material type that gets returned
    Material R;
        
    // modelData[0] = Specular
    // modelData[1] = Glow
    // modelData[2-5] = Body color
    // modelData[6-8] = Border color
    // modelData[9] = Stroke Weight
    // modelData[10] = Inverse Density
    // modelData[11] = Bounce constant
    String[] modelData = loadData("data/materials/" + M + ".aim");
    int body = color(Str2Int(modelData[2]), Str2Int(modelData[3]), Str2Int(modelData[4]), Str2Flt(modelData[5]));
    int border = color(Str2Int(modelData[6]), Str2Int(modelData[7]), Str2Int(modelData[8]));
    R = new Material(Str2Flt(modelData[0]), Str2Flt(modelData[1]), color(Str2Int(modelData[2]), Str2Int(modelData[3]), Str2Int(modelData[4]), Str2Flt(modelData[5])), color(Str2Int(modelData[6]), Str2Int(modelData[7]), Str2Int(modelData[8])), Str2Int(modelData[9]), Str2Flt(modelData[10]), Str2Flt(modelData[11])/100);
    
    return R;
  }
  
  /**
   * Convert a String to an Integer
   */
  int Str2Int(String S)
  {
    return Integer.parseInt(S);
  }
  
  /**
   * Convert a String to a Float
   */
  float Str2Flt(String S)
  {
    return (float) Integer.parseInt(S);
  }
}
