// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Model.java
// An abstract type for everything that exists in the game world. Characters, obstacles
// etc etc will all have to extend this abstract class and implement the abstract
// methods.
// ------------------------------------------------------------------------------

abstract class Model
{
  String name;
  String type;
  Polygon poly;
  Material material;
  Point center;
  Settings settings;
  ParticleManager particleManager;
  Stack toBeAdded;
  
  /**
   * Constructor
   */
  Model(String N, String T, Polygon P, Material M, Point C, float R, Settings S, String I)
  {
    name = N;
    type = T;
    poly = P.rotate(R);
    material = M;
    center = C;
    settings = S;
    particleManager = new ParticleManager();
    toBeAdded = new Stack();
    
    parseInput(I);
  }

  /**
   * Parses the input string into a
   * nice and readable array
   */
  String[] inputToArray(String input)
  {
    String[] R = input.split("&");
    for (int i=0; i<R.length; i++)
    {
      R[i] = R[i].split("=")[1];
    }
    return R;
  }
  
  /**
   * Convert a String to an Integer
   */
  int Str2Int(String S)
  {
    return Integer.parseInt(S);
  }
  
  /**
   * Convert a String to a Float
   */
  float Str2Flt(String S)
  {
    return (float) Integer.parseInt(S);
  }
  
  /**
   * Creates and destroys particles at correct
   * intervals with changes in rotation and speed
   */
  void emitParticles(String type, float rotation, Point center, Vector veloc, int number)
  {
    for (int i=0; i<number; i++)
    {
      Particle P;
      float rand = random(0.99,1.01);    // We need this to make each particle slightly unique
      
      if (type.equals("AccelHuman"))
      {
        P = new AccelHumanParticle(rotation, center, veloc.multiply(rand));
      }
      else
      {
        break;
      }
      
      particleManager.addParticle(P);
    }
  }
  
  /**
   * Adds an item to the list of 
   * models to be added to the world
   */
  void addModel(Model M)
  {
    toBeAdded.push(M);
  }
  
  /**
   * Returns the model that it 
   * wants to add to the gameworld.
   *
   * Only one model can be added by
   * any model at each frame.
   */
  Model modelToBeAdded()
  {
    if (toBeAdded.size() != 0)
    {
      return ((Model) toBeAdded.pop());
    }
    else
    {
      return null;
    }
  }
  
  /**
   * Give the agent that 'runs' the model
   * the information it needs to update itself
   * in this frame.
   */
  abstract Model frame(ArrayList models, boolean[] keys);
  
  /**
   * Each model should be able to
   * parse its settings string from
   * the game world.
   */
  abstract void parseInput(String inputs);
}
