// ------------------------------------------------------------------------------
// http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Polygon.java
// Data structure for storing polygons.
// ------------------------------------------------------------------------------

class Polygon
{
  Point[] vertices;
  float rotation;
  float radius;
  
  /**
   * Constructor with no arguments
   */
  Polygon()
  {
    vertices = new Point[0];
    rotation = 0;
  }
  
  /**
   * Constructor with set vertices
   */
  Polygon(Point[] Vertices)
  {
    vertices = Vertices;
    rotation = 0;
  }
  
  /**
   * Returns the number of vertices in
   * the polygon
   */
  int numVertex()
  {
    return vertices.length;
  }
  
  /**
   * Adds a vertex to the current vertex
   * list
   */
  void insertVertex(Point V)
  {
    Point[] verticesNew = new Point[numVertex()+1];
    
    for (int i=0; i<numVertex(); i++)
    {
      verticesNew[i] = vertices[i];
    }
    
    verticesNew[numVertex()] = V;
    vertices = verticesNew;
    
    float distance = sqrt(pow(V.x,2)+pow(V.y,2));
    if (distance > radius) radius = distance;
  }
  
  /**
   * Calculates the area of the polygon
   */
  float area()
  {
    float R = 0;
    for (int i=0; i<numVertex(); i++)
    {
      R += (vertices[i%numVertex()].x*vertices[(i+1)%numVertex()].y-vertices[(i+1)%numVertex()].x*vertices[i%numVertex()].y)/2;
    }
    return abs(R);
  }
  
  /**
   * Rotates the polygon
   */
  Polygon rotate(float theta)
  {
    Polygon R = new Polygon();
    
    R.rotation = rotation + theta;
    if (R.rotation > TWO_PI) R.rotation -= TWO_PI;
    if (R.rotation < 0 ) R.rotation += TWO_PI;
    
    for (int i=0; i<numVertex(); i++)
    {
      R.insertVertex(vertices[i].rotate(theta));
    }
    
    return R;
  }
  
  /**
   * Returns the number of edges
   * the polygon has
   */
  int numEdges()
  {
    return vertices.length;
  }
  
  /**
   * Returns a vector corresponding
   * to the correct edge
   */
  Vector getEdge(int index)
  {
    return new Vector(vertices[(index+1)%numEdges()].x-vertices[index%numEdges()].x,vertices[(index+1)%numEdges()].y-vertices[index%numEdges()].y);
  }
}
